@@ -34,6 +34,8 @@ namespace detail {
 
 struct gcc_sparc_cas_base
 {
+    static BOOST_CONSTEXPR_OR_CONST bool is_always_lock_free = true;
+
     static BOOST_FORCEINLINE void fence_before_store(memory_order order) BOOST_NOEXCEPT
     {
         if ((order & memory_order_release) != 0)
@@ -77,7 +79,7 @@ struct gcc_sparc_cas32 :
 
     static BOOST_FORCEINLINE void store(storage_type volatile& storage, storage_type v, memory_order order) BOOST_NOEXCEPT
     {
-        fence_before_store(order);
+        fence_before(order);
         storage = v;
         fence_after_store(order);
     }
@@ -118,7 +120,7 @@ struct gcc_sparc_cas32 :
 
     static BOOST_FORCEINLINE storage_type exchange(storage_type volatile& storage, storage_type v, memory_order order) BOOST_NOEXCEPT
     {
-        base_type::fence_before(order);
+        fence_before(order);
         __asm__ __volatile__
         (
             "swap [%1], %0"
@@ -126,14 +128,9 @@ struct gcc_sparc_cas32 :
             : "r" (&storage)
             : "memory"
         );
-        base_type::fence_after(order);
+        fence_after(order);
         return v;
     }
-
-    static BOOST_FORCEINLINE bool is_lock_free(storage_type const volatile&) BOOST_NOEXCEPT
-    {
-        return true;
-    }
 };
 
 template< bool Signed >
@@ -163,7 +160,7 @@ struct gcc_sparc_cas64 :
 
     static BOOST_FORCEINLINE void store(storage_type volatile& storage, storage_type v, memory_order order) BOOST_NOEXCEPT
     {
-        fence_before_store(order);
+        fence_before(order);
         storage = v;
         fence_after_store(order);
     }
@@ -201,11 +198,6 @@ struct gcc_sparc_cas64 :
     {
         return compare_exchange_strong(storage, expected, desired, success_order, failure_order);
     }
-
-    static BOOST_FORCEINLINE bool is_lock_free(storage_type const volatile&) BOOST_NOEXCEPT
-    {
-        return true;
-    }
 };
 
 template< bool Signed >
